require 'addressable/uri'
require 'json'
require 'rest_client'
require 'foxtrot/required_fields'
require 'foxtrot/response'
require 'foxtrot/errors'

module Foxtrot
  class Client

    class << self
      attr_accessor :host, :ssl
    end

    @host = 'app.foxtrot.io'
    @ssl = true

    def initialize(api_key, version='v1')
      @api_key = api_key
      @version = version
      @headers = {:content_type => :json, :accept => :json}
    end

    def poll!(endpoint, txid, args={})
      resp = get_request(endpoint, {txid: txid})
      Foxtrot::Response::Response.response_for(endpoint, resp, self)
    end

    def optimize!(data, args={})
      check_required 'optimize', data
      resp = post_request 'optimize', data, args
      Foxtrot::Response::Response.response_for('optimize', resp, self)
    end

    private

    def check_required(endpoint, data)
      required = Foxtrot::RequiredFields.const_get(endpoint.upcase)
      data = data.inject({}){ |h, (k,v)| h.merge({k.to_sym => v}) }
      required.each do |field|
        unless data.include?(field)
          raise Foxtrot::Errors::ParameterError.new(field)
        end
      end
    end

    def build_url(endpoint, args)
      uri = Addressable::URI.new
      uri.host = self.class.host
      uri.scheme = self.class.ssl ? 'https' : 'http'
      uri.query_values = args.merge({key: @api_key})
      uri.path = "/api/#{@version}/#{endpoint}"
      uri.to_s
    end

    def get_request(endpoint, args)
      resp = RestClient.get build_url(endpoint, args), @headers
      JSON.load resp
    end

    def post_request(endpoint, data, args)
      resp = RestClient.post build_url(endpoint, args), data.to_json, @headers
      JSON.load resp
    end
  end
end
