require 'foxtrot/errors'

module Foxtrot
  module Response
    class Response
      def initialize(endpoint, resp, api_instance)
        @endpoint = endpoint
        @resp = resp
        @api_instance = api_instance
      end

      def poll_and_block!(interval=1, timeout=60)
        count = 0
        instance = self
        while !instance.complete?
          instance = instance.poll!
          sleep interval
          count += interval
          if count > timeout
            raise Foxtrot::Errors::APITimeoutError.new(count)
          end
        end
        instance
      end

      def self.response_for(endpoint, resp, api_instance)
        case resp['status']
        when 'pending'
          PendingResponse.new(endpoint, resp, api_instance)
        when 'success'
          SuccessResponse.new(endpoint, resp, api_instance)
        else
          ErrorResponse.new(endpoint, resp, api_instance)
        end
      end
    end

    class ErrorResponse < Response
      def initialize(endpoint, resp, api_instance)
        super
        @message = resp['message']
        @exception = Foxtrot::Errors::APIResponseError.new @message
        raise @exception
      end

      def complete?
        true
      end

      def poll!
        raise @exception
      end

      def get_result
        nil
      end

      def to_s
        "ErrorResponse (#{@message})"
      end
    end

    class PendingResponse < Response
      def initialize(endpoint, resp, api_instance)
        super
        @txid = resp['txid']
      end

      def complete?
        false
      end

      def poll!
        @api_instance.poll! @endpoint, @txid
      end

      def get_result
        nil
      end

      def to_s
        "PendingResponse (#{@txid})"
      end
    end

    class SuccessResponse < Response
      def initialize(endpoint, resp, api_instance)
        super
        @txid = resp['txid']
        @data = resp['response']['api_data']
      end

      def complete?
        true
      end

      def poll!
        self
      end

      def get_result
        @data
      end

      def to_s
        "SuccessResponse (#{@txid})"
      end
    end
  end
end
